@@ -99,7 +99,7 @@ async def extract_tables(
             "size": stat.st_size,
             "mtime": stat.st_mtime,
         }
-    except OSError:
+    except OSError:  # pragma: no cover
         file_info = {
             "path": str(path),
             "size": 0,
@@ -193,9 +193,14 @@ async def extract_tables(
                 await run_sync(doc.close)
 
         except ImportError as e:  # pragma: no cover
-            raise MissingDependencyError.create_for_package(
+            error = MissingDependencyError.create_for_package(
                 dependency_group="gmft", functionality="table extraction", package_name="gmft"
-            ) from e
+            )
+            error.context = {
+                "file_path": str(Path(file_path)),
+                "error_message": str(e),
+            }
+            raise error from e
     finally:
         table_cache.mark_complete(**cache_kwargs)
 
@@ -215,7 +220,7 @@ def extract_tables_sync(
             "size": stat.st_size,
             "mtime": stat.st_mtime,
         }
-    except OSError:
+    except OSError:  # pragma: no cover
         file_info = {
             "path": str(path),
             "size": 0,
@@ -294,9 +299,14 @@ def extract_tables_sync(
             doc.close()  # type: ignore[no-untyped-call]
 
     except ImportError as e:  # pragma: no cover
-        raise MissingDependencyError.create_for_package(
+        error = MissingDependencyError.create_for_package(
             dependency_group="gmft", functionality="table extraction", package_name="gmft"
-        ) from e
+        )
+        error.context = {
+            "file_path": str(Path(file_path)),
+            "error_message": str(e),
+        }
+        raise error from e
 
 
 def _extract_tables_in_process(
@@ -312,6 +322,11 @@ def _extract_tables_in_process(
         from gmft.formatters.tatr import TATRFormatConfig  # noqa: PLC0415
         from gmft.pdf_bindings.pdfium import PyPDFium2Document  # noqa: PLC0415
 
+        if "cell_required_confidence" in config_dict:
+            cell_config = config_dict["cell_required_confidence"]
+            if isinstance(cell_config, dict) and cell_config:
+                config_dict["cell_required_confidence"] = {int(k): v for k, v in cell_config.items()}
+
         config = GMFTConfig(**config_dict)
 
         formatter = AutoTableFormatter(  # type: ignore[no-untyped-call]
@@ -444,6 +459,17 @@ def _extract_tables_isolated(
             return tables
 
         error_info = result
+        if error_info.get("type") == "ImportError":
+            error = MissingDependencyError.create_for_package(
+                dependency_group="gmft", functionality="table extraction", package_name="gmft"
+            )
+            error.context = {
+                "file_path": str(Path(file_path)),
+                "error_message": error_info["error"],
+                "traceback": error_info.get("traceback"),
+            }
+            raise error from ImportError(error_info["error"])
+
         raise ParsingError(
             f"GMFT table extraction failed: {error_info['error']}",
             context={
@@ -531,6 +557,17 @@ async def _extract_tables_isolated_async(
             return tables
 
         error_info = result
+        if error_info.get("type") == "ImportError":
+            error = MissingDependencyError.create_for_package(
+                dependency_group="gmft", functionality="table extraction", package_name="gmft"
+            )
+            error.context = {
+                "file_path": str(Path(file_path)),
+                "error_message": error_info["error"],
+                "traceback": error_info.get("traceback"),
+            }
+            raise error from ImportError(error_info["error"])
+
         raise ParsingError(
             f"GMFT table extraction failed: {error_info['error']}",
             context={
