@@ -3,6 +3,7 @@ from __future__ import annotations
 import csv
 import hashlib
 import io
+import logging
 import os
 import re
 import subprocess
@@ -14,12 +15,11 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, ClassVar, Final
 
 import anyio
-import html_to_markdown
 import polars as pl
 from anyio import Path as AsyncPath
 from anyio import run_process
-from bs4 import BeautifulSoup
-from bs4.element import Tag
+from html_to_markdown import HtmlToMarkdownError
+from html_to_markdown._html_to_markdown import convert as rust_convert
 from PIL import Image
 from PIL.Image import Image as PILImage
 from typing_extensions import Self
@@ -35,8 +35,9 @@ from kreuzberg._utils._sync import run_sync
 from kreuzberg._utils._tmp import create_temp_file, temporary_file_sync
 from kreuzberg.exceptions import MissingDependencyError, OCRError, ValidationError
 
+logger = logging.getLogger(__name__)
+
 if TYPE_CHECKING:
-    from bs4.element import Tag
     from PIL.Image import Image as PILImage
 
 try:  # pragma: no cover
@@ -214,7 +215,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
 
             try:
                 await run_sync(save_image.save, str(image_path), format="PNG")
-            except OSError as e:
+            except OSError as e:  # pragma: no cover
                 if "cannot write mode" not in str(e):
                     raise
                 save_image = image.convert("RGB")
@@ -300,8 +301,15 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             "OFF",
         ]
 
-        if run_config["tesseract_format"] != "text":
-            command.append(run_config["tesseract_format"])
+        # Handle output format - use config option for HOCR to ensure Windows compatibility
+        # Windows Tesseract 5.5.0 doesn't respect 'hocr' configfile, needs explicit config
+        tesseract_format = run_config["tesseract_format"]
+        if tesseract_format == "hocr":
+            command.extend(["-c", "tessedit_create_hocr=1"])
+        elif tesseract_format == "tsv":
+            command.append("tsv")
+        elif tesseract_format != "text":
+            command.append(tesseract_format)
 
         for kwarg, value in run_config["remaining_kwargs"].items():
             if kwarg.startswith("table_"):
@@ -356,7 +364,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
         try:
             stat = path.stat()
             file_info = {"path": str(path.resolve()), "size": stat.st_size, "mtime": stat.st_mtime}
-        except OSError:
+        except OSError:  # pragma: no cover
             file_info = {"path": str(path), "size": 0, "mtime": 0}
 
         cache_kwargs = {
@@ -398,7 +406,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                     await ocr_cache.aset(extraction_result, **final_cache_kwargs)
 
                 return extraction_result
-            except (RuntimeError, OSError) as e:
+            except (RuntimeError, OSError) as e:  # pragma: no cover
                 raise OCRError(f"Failed to OCR using tesseract: {e}") from e
             finally:
                 await unlink()
@@ -431,7 +439,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
 
                 try:
                     df = await run_sync(pl.DataFrame, table_data[1:], schema=table_data[0])
-                except (ImportError, IndexError):
+                except (ImportError, IndexError):  # pragma: no cover
                     df = None
 
                 table: TableData = {"text": markdown, "df": df, "page_number": 1, "cropped_image": None}  # type: ignore[typeddict-item]
@@ -443,7 +451,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                     tables=[table],
                     chunks=text_result.chunks,
                 )
-        except (ValueError, KeyError, ImportError):
+        except (ValueError, KeyError, ImportError):  # pragma: no cover
             pass
 
         return text_result
@@ -506,219 +514,56 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
         table_min_confidence: float = 30.0,
         **_kwargs: Any,
     ) -> ExtractionResult:
-        config = html_to_markdown_config or HTMLToMarkdownConfig(
-            escape_asterisks=False,
-            escape_underscores=False,
-            extract_metadata=False,
-            strip="meta title",
-        )
+        _ = (
+            enable_table_detection,
+            table_column_threshold,
+            table_row_threshold_ratio,
+            table_min_confidence,
+        )  # parameters retained for compatibility but handled internally by html-to-markdown
 
-        tables: list[TableData] = []
-        if enable_table_detection:
-            soup = BeautifulSoup(hocr_content, "xml")
-            tables = await self._extract_tables_from_hocr(
-                soup,
-                table_column_threshold,
-                table_row_threshold_ratio,
-                table_min_confidence,
-            )
-
-        hocr_converters = self._create_hocr_converters(tables)
-
-        all_converters = dict(hocr_converters)
-        if config.custom_converters:
-            all_converters.update(config.custom_converters)
-
-        config_dict = config.to_dict()
-        config_dict["custom_converters"] = all_converters
+        config = html_to_markdown_config or HTMLToMarkdownConfig()
+        conversion_options, _ = config.to_options()
 
         try:
-            markdown_content = html_to_markdown.convert_to_markdown(hocr_content, **config_dict)
+            markdown_content = rust_convert(
+                hocr_content,
+                conversion_options,
+            )
             markdown_content = normalize_spaces(markdown_content)
-        except (ValueError, TypeError, AttributeError):
-            try:
-                soup = BeautifulSoup(hocr_content, "xml")
-                words = soup.find_all("span", class_="ocrx_word")
-                text_parts = []
-                for word in words:
-                    text = word.get_text().strip()
-                    if text:
-                        text_parts.append(text)
+        except (HtmlToMarkdownError, ValueError) as exc:
+            logger.exception("Failed to convert hOCR to Markdown: %s", exc)
+            markdown_content = "[OCR processing failed]"
 
-                if text_parts:
-                    markdown_content = " ".join(text_parts)
-                else:
-                    markdown_content = soup.get_text().strip() or "[No text detected]"
-
-                markdown_content = normalize_spaces(markdown_content)
-            except (ValueError, TypeError, AttributeError):
-                markdown_content = "[OCR processing failed]"
-
-        if tables:
-            table_sections = []
-            for i, table in enumerate(tables):
-                table_sections.append(f"\n## Table {i + 1}\n\n{table['text']}\n")
-
-            if markdown_content.strip():
-                final_content = f"{markdown_content}\n{''.join(table_sections)}"
-            else:
-                final_content = "".join(table_sections).strip()
-        else:
-            final_content = markdown_content
+        tables: list[TableData] = []
 
         return ExtractionResult(
-            content=final_content,
+            content=markdown_content,
             mime_type=MARKDOWN_MIME_TYPE,
             metadata={"source_format": "hocr", "tables_detected": len(tables)},
             chunks=[],
             tables=tables,
         )
 
-    def _create_basic_converters(self) -> dict[str, Any]:
-        def ocrx_word_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag
-            return f"{text.strip()} "
-
-        def ocr_line_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag
-            return f"{text.strip()}\n"
-
-        def ocr_par_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag
-            content = text.strip()
-            if not content:
-                return ""
-            return f"{content}\n\n"
-
-        def ocr_carea_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag
-            content = text.strip()
-            if not content:
-                return ""
-            return f"{content}\n\n"
-
-        def ocr_page_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag
-            return text.strip()
-
-        def ocr_separator_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del tag, text
-            return "---\n"
-
-        def ocr_photo_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            del text
-            title = tag.get("title", "")
-            if isinstance(title, str):
-                bbox_match = re.search(r"bbox (\d+) (\d+) (\d+) (\d+)", title)
-                if bbox_match:
-                    x0, y0, x1, y1 = bbox_match.groups()
-                    width = int(x1) - int(x0)
-                    height = int(y1) - int(y0)
-                    return f"*[Image region: {width}x{height} pixels]*\n\n"
-            return "*[Image detected]*\n\n"
-
-        return {
-            "ocrx_word": ocrx_word_converter,
-            "ocr_line": ocr_line_converter,
-            "ocr_par": ocr_par_converter,
-            "ocr_carea": ocr_carea_converter,
-            "ocr_page": ocr_page_converter,
-            "ocr_separator": ocr_separator_converter,
-            "ocr_photo": ocr_photo_converter,
-        }
-
-    def _create_hocr_converters(self, _tables: list[TableData]) -> dict[str, Any]:
-        basic_converters = self._create_basic_converters()
-
-        def generic_div_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            class_attr = tag.get("class", "")
-            if isinstance(class_attr, list):
-                class_attr = " ".join(class_attr)
-            elif not isinstance(class_attr, str):
-                class_attr = ""
-
-            for class_name in ["ocr_separator", "ocr_photo", "ocr_page", "ocr_carea"]:
-                if class_name in class_attr:
-                    converter_result = basic_converters[class_name](tag=tag, text=text, **_conv_kwargs)
-                    return str(converter_result)
-            return text
-
-        def generic_span_converter(*, tag: Tag, text: str, **_conv_kwargs: Any) -> str:
-            class_attr = tag.get("class", "")
-            if isinstance(class_attr, list):
-                class_attr = " ".join(class_attr)
-            elif not isinstance(class_attr, str):
-                class_attr = ""
-
-            for class_name in ["ocrx_word", "ocr_line"]:
-                if class_name in class_attr:
-                    converter_result = basic_converters[class_name](tag=tag, text=text, **_conv_kwargs)
-                    return str(converter_result)
-            return f"{text.strip()} "
-
-        return {
-            "span": generic_span_converter,
-            "div": generic_div_converter,
-            "p": basic_converters["ocr_par"],
-        }
-
     def _process_hocr_to_markdown_sync(self, hocr_content: str, config: TesseractConfig) -> ExtractionResult:
-        tables: list[TableData] = []
+        _ = config  # retained for interface compatibility
 
-        if config.enable_table_detection:
-            pass
+        html_config = HTMLToMarkdownConfig()
+        conversion_options, _ = html_config.to_options()
 
         try:
-            converters = self._create_hocr_converters(tables)
-
-            html_config = HTMLToMarkdownConfig(
-                custom_converters=converters,
-                escape_asterisks=False,
-                escape_underscores=False,
-                extract_metadata=False,
-                strip="meta title",
-            )
-
-            markdown_content = html_to_markdown.convert_to_markdown(
+            markdown_content = rust_convert(
                 hocr_content,
-                **html_config.to_dict(),
+                conversion_options,
             )
-
             markdown_content = normalize_spaces(markdown_content)
+        except (HtmlToMarkdownError, ValueError) as exc:
+            logger.exception("Failed to convert hOCR to Markdown (sync path): %s", exc)
+            markdown_content = "[OCR processing failed]"
 
-        except (ValueError, TypeError, AttributeError):
-            try:
-                soup = BeautifulSoup(hocr_content, "xml")
-                words = soup.find_all("span", class_="ocrx_word")
-                text_parts = []
-                for word in words:
-                    text = word.get_text().strip()
-                    if text:
-                        text_parts.append(text)
-
-                if text_parts:
-                    markdown_content = " ".join(text_parts)
-                else:
-                    markdown_content = soup.get_text().strip() or "[No text detected]"
-
-                markdown_content = normalize_spaces(markdown_content)
-            except (ValueError, TypeError, AttributeError):
-                markdown_content = "[OCR processing failed]"
-
-        if tables:
-            table_sections = []
-            for i, table in enumerate(tables):
-                table_sections.append(f"\n## Table {i + 1}\n\n{table['text']}\n")
-
-            if markdown_content.strip():
-                final_content = f"{markdown_content}\n{''.join(table_sections)}"
-            else:
-                final_content = "".join(table_sections).strip()
-        else:
-            final_content = markdown_content
+        tables: list[TableData] = []
 
         return ExtractionResult(
-            content=final_content,
+            content=markdown_content,
             mime_type=MARKDOWN_MIME_TYPE,
             metadata={"source_format": "hocr", "tables_detected": len(tables)},
             chunks=[],
@@ -750,7 +595,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
 
                 try:
                     df = pl.DataFrame(table_data[1:], schema=table_data[0])
-                except (ImportError, IndexError):
+                except (ImportError, IndexError):  # pragma: no cover
                     df = None
 
                 table: TableData = {"text": markdown, "df": df, "page_number": 1, "cropped_image": None}  # type: ignore[typeddict-item]
@@ -762,102 +607,11 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                     tables=[table],
                     chunks=text_result.chunks,
                 )
-        except (ValueError, KeyError, ImportError):
+        except (ValueError, KeyError, ImportError):  # pragma: no cover
             pass
 
         return text_result
 
-    async def _extract_tables_from_hocr(
-        self,
-        soup: Any,
-        column_threshold: int = 20,
-        row_threshold_ratio: float = 0.5,
-        min_confidence: float = 30.0,
-    ) -> list[TableData]:
-        tsv_data = await self._hocr_to_tsv_data(soup, min_confidence)
-
-        if not tsv_data:
-            return []
-
-        if not (words := extract_words(tsv_data, min_confidence=min_confidence)):
-            return []
-
-        tables: list[TableData] = []
-        try:
-            table_data = reconstruct_table(
-                words,
-                column_threshold=column_threshold,
-                row_threshold_ratio=row_threshold_ratio,
-            )
-            if table_data and len(table_data) > 1:  # ~keep At least header + one data row
-                markdown = to_markdown(table_data)
-
-                min_x = min(w["left"] for w in words)
-                max_x = max(w["left"] + w["width"] for w in words)
-                min_y = min(w["top"] for w in words)
-                max_y = max(w["top"] + w["height"] for w in words)
-
-                try:
-                    df = await run_sync(pl.DataFrame, table_data[1:], schema=table_data[0])
-                except (ImportError, IndexError):
-                    df = None
-
-                dummy_image = Image.new("RGB", (1, 1), "white")
-
-                table: TableData = {
-                    "text": markdown,
-                    "df": df,
-                    "page_number": 1,
-                    "cropped_image": dummy_image,
-                    "metadata": {"bbox": (min_x, min_y, max_x, max_y)},
-                }  # type: ignore[typeddict-unknown-key]
-                tables.append(table)
-        except (ValueError, KeyError, ImportError):
-            pass
-
-        return tables
-
-    async def _hocr_to_tsv_data(self, soup: Any, min_confidence: float) -> str:
-        tsv_lines = ["level\tpage_num\tblock_num\tpar_num\tline_num\tword_num\tleft\ttop\twidth\theight\tconf\ttext"]
-
-        words = soup.find_all("span", class_="ocrx_word")
-        word_num = 1
-
-        for word in words:
-            title = word.get("title", "")
-            text = word.get_text().strip()
-
-            if not text:
-                continue
-
-            bbox_match = re.search(r"bbox (\d+) (\d+) (\d+) (\d+)", title)
-            if not bbox_match:
-                continue
-
-            x0, y0, x1, y1 = map(int, bbox_match.groups())
-
-            conf_match = re.search(r"x_wconf (\d+)", title)
-            confidence = float(conf_match.group(1)) if conf_match else 100.0
-
-            if confidence < min_confidence:
-                continue
-
-            line = word.find_parent(class_="ocr_line")
-            par = word.find_parent(class_="ocr_par")
-            block = word.find_parent(class_="ocr_carea")
-
-            tsv_line = f"5\t1\t{block.get('id', '1').split('_')[-1] if block else 1}\t{par.get('id', '1').split('_')[-1] if par else 1}\t{line.get('id', '1').split('_')[-1] if line else 1}\t{word_num}\t{x0}\t{y0}\t{x1 - x0}\t{y1 - y0}\t{confidence}\t{text}"
-            tsv_lines.append(tsv_line)
-            word_num += 1
-
-        return "\n".join(tsv_lines)
-
-    def _identify_table_regions(self, words: list[dict[str, Any]]) -> list[list[dict[str, Any]]]:
-        if not words:
-            return []
-
-        return [words]
-
     @classmethod
     async def _validate_tesseract_version(cls) -> None:
         try:
@@ -868,7 +622,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             env = {"OMP_THREAD_LIMIT": "1"} if sys.platform.startswith("linux") else None
             try:
                 result = await run_process(command, env=env)
-            except (subprocess.CalledProcessError, FileNotFoundError) as e:
+            except (subprocess.CalledProcessError, FileNotFoundError) as e:  # pragma: no cover
                 raise MissingDependencyError(
                     "Tesseract version 5 is a required system dependency. Please install it on your system and make sure its available in $PATH."
                 ) from e
@@ -879,7 +633,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                 )
 
             cls._version_checked = True
-        except FileNotFoundError as e:
+        except FileNotFoundError as e:  # pragma: no cover
             raise MissingDependencyError(
                 "Tesseract version 5 is a required system dependency. Please install it on your system and make sure its available in $PATH."
             ) from e
@@ -1076,7 +830,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                 "size": stat.st_size,
                 "mtime": stat.st_mtime,
             }
-        except OSError:
+        except OSError:  # pragma: no cover
             return {
                 "path": str(path),
                 "size": 0,
@@ -1084,7 +838,6 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             }
 
     def _result_from_dict(self, result_dict: dict[str, Any]) -> ExtractionResult:
-        """Convert a worker result dict to ExtractionResult."""
         if result_dict.get("success"):
             return ExtractionResult(
                 content=str(result_dict.get("text", "")),
@@ -1112,6 +865,10 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             **run_config["remaining_kwargs"],
             "language": run_config["language"],
             "psm": run_config["psm"],
+            "tesseract_format": run_config["tesseract_format"],
+            "ext": run_config["ext"],
+            "output_format": run_config["output_format"],
+            "enable_table_detection": run_config["enable_table_detection"],
         }
 
         optimal_workers = get_optimal_worker_count(len(paths), cpu_intensive=True)
@@ -1157,7 +914,13 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             "OFF",
         ]
 
-        if output_format != "text":
+        # Handle output format - use config option for HOCR to ensure Windows compatibility
+        # Windows Tesseract 5.5.0 doesn't respect 'hocr' configfile, needs explicit config
+        if output_format == "hocr":
+            command.extend(["-c", "tessedit_create_hocr=1"])
+        elif output_format == "tsv":
+            command.append("tsv")
+        elif output_format != "text":
             command.append(output_format)
 
         for kwarg, value in kwargs.items():
@@ -1178,7 +941,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
             command = ["tesseract", "--version"]
             try:
                 result = subprocess.run(command, capture_output=True, text=True, check=True, encoding="utf-8")
-            except (subprocess.CalledProcessError, FileNotFoundError) as e:
+            except (subprocess.CalledProcessError, FileNotFoundError) as e:  # pragma: no cover
                 raise MissingDependencyError(
                     "Tesseract version 5 is a required system dependency. Please install it on your system and make sure its available in $PATH."
                 ) from e
@@ -1189,7 +952,7 @@ class TesseractBackend(OCRBackend[TesseractConfig]):
                 )
 
             cls._version_checked = True
-        except FileNotFoundError as e:
+        except FileNotFoundError as e:  # pragma: no cover
             raise MissingDependencyError(
                 "Tesseract version 5 is a required system dependency. Please install it on your system and make sure its available in $PATH."
             ) from e
@@ -1221,13 +984,21 @@ def _process_image_with_tesseract(
     config_dict: dict[str, Any],
 ) -> dict[str, Any]:
     try:
-        with tempfile.NamedTemporaryFile(suffix=".txt", delete=False) as tmp_file:
-            output_base = tmp_file.name.replace(".txt", "")
+        tesseract_format = config_dict.get("tesseract_format", "text")
+        ext = config_dict.get("ext", ".txt")
+        output_format = config_dict.get("output_format", "text")
+        config_dict.get("enable_table_detection", False)
+
+        with tempfile.NamedTemporaryFile(suffix=ext, delete=False) as tmp_file:
+            output_base = tmp_file.name.replace(ext, "")
 
         try:
             language = config_dict.get("language", "eng")
             psm = config_dict.get("psm", 3)
 
+            # Convert PSM enum to integer value if needed
+            psm_value = psm.value if hasattr(psm, "value") else psm
+
             command = [
                 "tesseract",
                 image_path,
@@ -1235,13 +1006,16 @@ def _process_image_with_tesseract(
                 "-l",
                 language,
                 "--psm",
-                str(psm),
+                str(psm_value),
                 "--oem",
                 "1",
                 "--loglevel",
                 "OFF",
             ]
 
+            if tesseract_format != "text":
+                command.append(tesseract_format)
+
             boolean_options = [
                 "classify_use_pre_adapted_templates",
                 "language_model_ngram_on",
@@ -1274,10 +1048,16 @@ def _process_image_with_tesseract(
             if result.returncode != 0:
                 raise Exception(f"Tesseract failed with return code {result.returncode}: {result.stderr}")
 
-            output_file = output_base + ".txt"
+            output_file = output_base + ext
             with Path(output_file).open(encoding="utf-8") as f:
                 text = f.read()
 
+            # Process based on output format
+            if output_format == "markdown" and tesseract_format == "hocr":
+                html_config = HTMLToMarkdownConfig(heading_style="atx")
+                options, _ = html_config.to_options()
+                text = rust_convert(text, options)
+
             text = normalize_spaces(text)
 
             return {
@@ -1288,8 +1068,8 @@ def _process_image_with_tesseract(
             }
 
         finally:
-            for ext in [".txt"]:
-                temp_file = output_base + ext
+            for possible_ext in [ext, ".txt", ".hocr", ".tsv"]:
+                temp_file = output_base + possible_ext
                 temp_path = Path(temp_file)
                 if temp_path.exists():
                     temp_path.unlink()
