@@ -1,16 +1,26 @@
 from __future__ import annotations
 
+import json
 import sys
-from collections.abc import Awaitable, Callable, Iterable, Mapping
+import warnings
+from collections.abc import Awaitable, Callable, Mapping
 from dataclasses import asdict, dataclass, field
 from enum import Enum
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Literal, NamedTuple, TypedDict
 
+import langcodes
 import msgspec
+from html_to_markdown._html_to_markdown import (
+    ConversionOptions as HTMLToMarkdownConversionOptions,
+)
+from html_to_markdown._html_to_markdown import (
+    PreprocessingOptions as HTMLToMarkdownPreprocessingOptions,
+)
 
 from kreuzberg._constants import DEFAULT_MAX_CHARACTERS, DEFAULT_MAX_OVERLAP
 from kreuzberg._utils._table import (
+    enhance_table_markdown,
     export_table_to_csv,
     export_table_to_tsv,
     extract_table_structure_info,
@@ -31,6 +41,7 @@ if TYPE_CHECKING:
 
 OcrBackendType = Literal["tesseract", "easyocr", "paddleocr"]
 OutputFormatType = Literal["text", "tsv", "hocr", "markdown"]
+ErrorContextType = Literal["batch_processing", "optional_feature", "single_extraction", "unknown"]
 
 
 class ConfigDict:
@@ -254,6 +265,15 @@ class PaddleOCRConfig(ConfigDict):
 
 @dataclass(unsafe_hash=True, frozen=True, slots=True)
 class GMFTConfig(ConfigDict):
+    def __post_init__(self) -> None:
+        warnings.warn(
+            "GMFTConfig is deprecated and will be removed in Kreuzberg v4.0. "
+            "Install `kreuzberg[gmft]` only if you still rely on GMFT. "
+            "Future versions use native TATR-based table extraction via TableExtractionConfig.",
+            FutureWarning,
+            stacklevel=2,
+        )
+
     verbosity: int = 0
     """
     Verbosity level for logging.
@@ -401,9 +421,12 @@ class ImageOCRConfig(ConfigDict):
 
 @dataclass(unsafe_hash=True, frozen=True, slots=True)
 class LanguageDetectionConfig(ConfigDict):
-    low_memory: bool = True
-    """If True, uses a smaller model (~200MB). If False, uses a larger, more accurate model.
-    Defaults to True for better memory efficiency."""
+    model: Literal["lite", "full", "auto"] = "auto"
+    """Language detection model to use:
+    - 'lite': Smaller, faster model with good accuracy
+    - 'full': Larger model with highest accuracy
+    - 'auto': Automatically choose based on memory availability (default)
+    """
     top_k: int = 3
     """Maximum number of languages to return for multilingual detection."""
     multilingual: bool = False
@@ -411,8 +434,8 @@ class LanguageDetectionConfig(ConfigDict):
     If False, uses single language detection."""
     cache_dir: str | None = None
     """Custom directory for model cache. If None, uses system default."""
-    allow_fallback: bool = True
-    """If True, falls back to small model if large model fails."""
+    low_memory: bool = True
+    """Deprecated. Use 'model' parameter instead. If True, uses 'lite' model."""
 
 
 @dataclass(unsafe_hash=True, frozen=True, slots=True)
@@ -499,6 +522,17 @@ class SpacyEntityExtractionConfig(ConfigDict):
         return "xx_ent_wiki_sm" if self.fallback_to_multilingual else None
 
 
+class ProcessingErrorDict(TypedDict):
+    feature: str
+    """Name of the feature that failed (e.g., 'chunking', 'entity_extraction', 'keyword_extraction')."""
+    error_type: str
+    """Type of the exception that occurred (e.g., 'RuntimeError', 'ValidationError')."""
+    error_message: str
+    """Human-readable error message."""
+    traceback: str
+    """Full Python traceback for debugging."""
+
+
 class BoundingBox(TypedDict):
     left: int
     """X coordinate of the left edge."""
@@ -591,6 +625,8 @@ class ImagePreprocessingMetadata(NamedTuple):
 
 
 class Metadata(TypedDict, total=False):
+    abstract: NotRequired[str]
+    """Document abstract or summary."""
     authors: NotRequired[list[str]]
     """List of document authors."""
     categories: NotRequired[list[str]]
@@ -677,9 +713,32 @@ class Metadata(TypedDict, total=False):
     """Error message if extraction failed."""
     error_context: NotRequired[dict[str, Any]]
     """Error context information for debugging."""
+    json_schema: NotRequired[dict[str, Any]]
+    """JSON schema information extracted from structured data."""
+    notes: NotRequired[list[str]]
+    """Notes or additional information extracted from documents."""
+    note: NotRequired[str]
+    """Single note or annotation."""
+    name: NotRequired[str]
+    """Name field from structured data."""
+    body: NotRequired[str]
+    """Body text content."""
+    text: NotRequired[str]
+    """Generic text content."""
+    message: NotRequired[str]
+    """Message or communication content."""
+    attributes: NotRequired[dict[str, Any]]
+    """Additional attributes extracted from structured data (e.g., custom text fields with dotted keys)."""
+    token_reduction: NotRequired[dict[str, float]]
+    """Token reduction statistics including reduction ratios and counts."""
+    processing_errors: NotRequired[list[ProcessingErrorDict]]
+    """List of processing errors that occurred during extraction."""
+    extraction_error: NotRequired[dict[str, Any]]
+    """Error information for critical extraction failures."""
 
 
 _VALID_METADATA_KEYS = {
+    "abstract",
     "authors",
     "categories",
     "citations",
@@ -722,6 +781,17 @@ _VALID_METADATA_KEYS = {
     "source_format",
     "error",
     "error_context",
+    "json_schema",
+    "notes",
+    "note",
+    "name",
+    "body",
+    "text",
+    "message",
+    "attributes",
+    "token_reduction",
+    "processing_errors",
+    "extraction_error",
 }
 
 
@@ -730,9 +800,29 @@ def normalize_metadata(data: dict[str, Any] | None) -> Metadata:
         return {}
 
     normalized: Metadata = {}
+    attributes: dict[str, Any] = {}
+
     for key, value in data.items():
-        if key in _VALID_METADATA_KEYS and value is not None:
-            normalized[key] = value  # type: ignore[literal-required]
+        if value is not None:
+            if key in _VALID_METADATA_KEYS:
+                normalized[key] = value  # type: ignore[literal-required]
+            elif "." in key and key.split(".")[-1] in {
+                "title",
+                "name",
+                "subject",
+                "description",
+                "content",
+                "body",
+                "text",
+                "message",
+                "note",
+                "abstract",
+                "summary",
+            }:
+                attributes[key] = value
+
+    if attributes:
+        normalized["attributes"] = attributes
 
     return normalized
 
@@ -830,11 +920,61 @@ class ExtractionResult:
 
         return [extract_table_structure_info(table) for table in self.tables]
 
+    def to_markdown(self, show_metadata: bool = False) -> str:
+        """Render the extraction result as a Markdown document."""
+        sections: list[str] = []
+
+        content_block = self.content.rstrip()
+        if content_block:
+            sections.append(content_block)
+
+        if self.tables:
+            table_sections: list[str] = ["## Tables"]
+            for index, table in enumerate(self.tables, start=1):
+                table_heading = f"### Table {index}"
+                table_markdown = enhance_table_markdown(table).strip()
+                if not table_markdown and table.get("text"):
+                    table_markdown = str(table.get("text", "")).strip()
+                if not table_markdown:
+                    table_markdown = "_No table data available._"
+                table_sections.append(f"{table_heading}\n\n{table_markdown}")
+            sections.append("\n\n".join(table_sections))
+
+        if show_metadata and self.metadata:
+            metadata_payload = json.dumps(self.metadata, indent=2, ensure_ascii=False)
+            sections.append(f"## Metadata\n\n```json\n{metadata_payload}\n```")
+
+        return "\n\n".join(part.strip() for part in sections if part.strip())
+
 
 PostProcessingHook = Callable[[ExtractionResult], ExtractionResult | Awaitable[ExtractionResult]]
 ValidationHook = Callable[[ExtractionResult], None | Awaitable[None]]
 
 
+@dataclass(unsafe_hash=True, frozen=True, slots=True)
+class JSONExtractionConfig(ConfigDict):
+    extract_schema: bool = False
+    """Extract and include JSON schema information in metadata."""
+    custom_text_field_patterns: frozenset[str] | None = None
+    """Custom patterns to identify text fields beyond default keywords."""
+    max_depth: int = 10
+    """Maximum nesting depth to process in JSON structures."""
+    array_item_limit: int = 1000
+    """Maximum number of array items to process to prevent memory issues."""
+    include_type_info: bool = False
+    """Include data type information in extracted content."""
+    flatten_nested_objects: bool = True
+    """Flatten nested objects using dot notation for better text extraction."""
+
+    def __post_init__(self) -> None:
+        if self.max_depth <= 0:
+            raise ValidationError("max_depth must be positive", context={"max_depth": self.max_depth})
+        if self.array_item_limit <= 0:
+            raise ValidationError(
+                "array_item_limit must be positive", context={"array_item_limit": self.array_item_limit}
+            )
+
+
 @dataclass(unsafe_hash=True, frozen=True, slots=True)
 class ExtractionConfig(ConfigDict):
     force_ocr: bool = False
@@ -908,8 +1048,14 @@ class ExtractionConfig(ConfigDict):
     """Custom entity patterns as a frozenset of (entity_type, regex_pattern) tuples."""
     auto_detect_language: bool = False
     """Whether to automatically detect language and configure OCR accordingly."""
+    language_detection_model: Literal["lite", "full", "auto"] = "auto"
+    """Language detection model to use when auto_detect_language is True.
+    - 'lite': Smaller, faster model with good accuracy
+    - 'full': Larger model with highest accuracy
+    - 'auto': Automatically choose based on memory availability (default)
+    """
     language_detection_config: LanguageDetectionConfig | None = None
-    """Configuration for language detection. If None, uses default settings."""
+    """Configuration for language detection. If None, uses default settings with language_detection_model."""
     spacy_entity_extraction_config: SpacyEntityExtractionConfig | None = None
     """Configuration for spaCy entity extraction. If None, uses default settings."""
     auto_detect_document_type: bool = False
@@ -924,6 +1070,8 @@ class ExtractionConfig(ConfigDict):
     """Password(s) for encrypted PDF files. Can be a single password or list of passwords to try in sequence. Only used when crypto extra is installed."""
     html_to_markdown_config: HTMLToMarkdownConfig | None = None
     """Configuration for HTML to Markdown conversion. If None, uses default settings."""
+    json_config: JSONExtractionConfig | None = None
+    """Configuration for enhanced JSON extraction features. If None, uses standard JSON processing."""
     use_cache: bool = True
     """Whether to use caching for extraction results. Set to False to disable all caching."""
     target_dpi: int = 150
@@ -936,6 +1084,8 @@ class ExtractionConfig(ConfigDict):
     """Minimum DPI threshold when auto-adjusting DPI."""
     max_dpi: int = 600
     """Maximum DPI threshold when auto-adjusting DPI."""
+    token_reduction: TokenReductionConfig | None = None
+    """Configuration for token reduction to optimize output size while preserving meaning."""
 
     def __post_init__(self) -> None:
         if self.custom_entity_patterns is not None and isinstance(self.custom_entity_patterns, dict):
@@ -1060,71 +1210,167 @@ class ExtractionConfig(ConfigDict):
 
 @dataclass(unsafe_hash=True, frozen=True, slots=True)
 class HTMLToMarkdownConfig:
-    stream_processing: bool = False
-    """Enable streaming mode for processing large HTML documents."""
-    chunk_size: int = 1024
-    """Size of chunks when stream_processing is enabled."""
-    chunk_callback: Callable[[str], None] | None = None
-    """Callback function invoked for each chunk during stream processing."""
-    progress_callback: Callable[[int, int], None] | None = None
-    """Callback function for progress updates (current, total)."""
-    parser: str | None = "lxml"
-    """BeautifulSoup parser to use. Defaults to 'lxml' for ~30% better performance. Falls back to 'html.parser' if lxml not available."""
-    autolinks: bool = True
-    """Convert URLs to clickable links automatically."""
+    heading_style: Literal["underlined", "atx", "atx_closed"] = "atx"
+    """Style for markdown headings."""
+    list_indent_type: Literal["spaces", "tabs"] = "spaces"
+    """Type of indentation to use for lists."""
+    list_indent_width: int = 4
+    """Number of spaces per indentation level (use 2 for Discord/Slack)."""
     bullets: str = "*+-"
     """Characters to use for unordered list bullets."""
-    code_language: str = ""
-    """Default language for code blocks."""
-    code_language_callback: Callable[[Any], str] | None = None
-    """Callback to determine code language dynamically."""
-    convert: str | Iterable[str] | None = None
-    """HTML tags to convert. If None, all supported tags are converted."""
-    convert_as_inline: bool = False
-    """Convert block elements as inline elements."""
-    custom_converters: Mapping[Any, Any] | None = None
-    """Custom converters for specific HTML elements."""
-    default_title: bool = False
-    """Use a default title if none is found."""
-    escape_asterisks: bool = True
-    """Escape asterisks in text to prevent unintended emphasis."""
-    escape_misc: bool = True
-    """Escape miscellaneous characters that have special meaning in Markdown."""
-    escape_underscores: bool = True
-    """Escape underscores in text to prevent unintended emphasis."""
-    extract_metadata: bool = True
-    """Extract metadata from HTML head section."""
-    heading_style: Literal["underlined", "atx", "atx_closed"] = "underlined"
-    """Style for markdown headings."""
-    highlight_style: Literal["double-equal", "html", "bold"] = "double-equal"
-    """Style for highlighting text."""
-    keep_inline_images_in: Iterable[str] | None = None
-    """HTML tags where inline images should be preserved."""
-    newline_style: Literal["spaces", "backslash"] = "spaces"
-    """Style for line breaks in markdown."""
-    strip: str | Iterable[str] | None = None
-    """HTML tags to strip completely from output."""
-    strip_newlines: bool = False
-    """Strip newlines from the output."""
     strong_em_symbol: Literal["*", "_"] = "*"
     """Symbol to use for strong/emphasis formatting."""
+    escape_asterisks: bool = False
+    """Escape * characters to prevent unintended formatting."""
+    escape_underscores: bool = False
+    """Escape _ characters to prevent unintended formatting."""
+    escape_misc: bool = False
+    """Escape miscellaneous characters to prevent Markdown conflicts."""
+    escape_ascii: bool = False
+    """Escape all ASCII punctuation."""
+    code_language: str = ""
+    """Default language identifier for fenced code blocks."""
+    code_language_callback: Callable[[Any], str] | None = field(default=None, compare=False, hash=False)
+    """Legacy language callback (no longer used by v2 converter)."""
+    autolinks: bool = True
+    """Automatically convert valid URLs to Markdown links."""
+    default_title: bool = False
+    """Use default titles for elements like links."""
+    keep_inline_images_in: tuple[str, ...] | None = None
+    """Tags where inline images should be preserved."""
+    br_in_tables: bool = False
+    """Use <br> tags for line breaks in table cells instead of spaces."""
+    highlight_style: Literal["double-equal", "html", "bold", "none"] = "double-equal"
+    """Style for highlighting text."""
+    extract_metadata: bool = True
+    """Extract document metadata as comment header."""
+    whitespace_mode: Literal["normalized", "strict"] = "normalized"
+    """Whitespace handling mode."""
+    strip_newlines: bool = False
+    """Remove newlines from HTML input before processing."""
+    wrap: bool = False
+    """Enable text wrapping."""
+    wrap_width: int = 80
+    """Width for text wrapping."""
+    convert_as_inline: bool = False
+    """Treat content as inline elements only."""
     sub_symbol: str = ""
     """Symbol to use for subscript text."""
     sup_symbol: str = ""
     """Symbol to use for superscript text."""
-    wrap: bool = False
-    """Enable text wrapping."""
-    wrap_width: int = 80
-    """Width for text wrapping when wrap is True."""
-    preprocess_html: bool = True
-    """Enable HTML preprocessing to clean up the input."""
-    preprocessing_preset: Literal["minimal", "standard", "aggressive"] = "aggressive"
+    newline_style: Literal["spaces", "backslash"] = "spaces"
+    """Style for line breaks in markdown."""
+    code_block_style: Literal["indented", "backticks", "tildes"] = "backticks"
+    """Style for fenced code blocks."""
+    strip_tags: tuple[str, ...] | None = None
+    """List of HTML tags to remove from output."""
+    convert: tuple[str, ...] | None = None
+    """Legacy list of tags to convert (no longer used by v2 converter)."""
+    custom_converters: Mapping[str, Callable[..., str]] | None = field(default=None, compare=False, hash=False)
+    """Legacy mapping of custom converters (ignored by v2 converter)."""
+    preprocess_html: bool = False
+    """Enable HTML preprocessing to clean messy HTML."""
+    preprocessing_preset: Literal["minimal", "standard", "aggressive"] = "standard"
     """Preprocessing level for cleaning HTML."""
     remove_navigation: bool = True
-    """Remove navigation elements from HTML."""
+    """Remove navigation elements during preprocessing."""
     remove_forms: bool = True
-    """Remove form elements from HTML."""
+    """Remove form elements during preprocessing."""
+    encoding: str = "utf-8"
+    """Expected character encoding for the HTML input."""
+    debug: bool = False
+    """Enable debug diagnostics in the converter."""
 
-    def to_dict(self) -> dict[str, Any]:
+    def __post_init__(self) -> None:
+        if self.keep_inline_images_in is not None and not isinstance(self.keep_inline_images_in, tuple):
+            object.__setattr__(self, "keep_inline_images_in", tuple(self.keep_inline_images_in))
+        if self.strip_tags is not None and not isinstance(self.strip_tags, tuple):
+            object.__setattr__(self, "strip_tags", tuple(self.strip_tags))
+        if self.convert is not None and not isinstance(self.convert, tuple):
+            object.__setattr__(self, "convert", tuple(self.convert))
+
+    def to_options(self) -> tuple[HTMLToMarkdownConversionOptions, HTMLToMarkdownPreprocessingOptions]:
+        """Build html_to_markdown ConversionOptions and PreprocessingOptions instances."""
+        preprocessing = HTMLToMarkdownPreprocessingOptions(
+            enabled=self.preprocess_html,
+            preset=self.preprocessing_preset,
+            remove_navigation=self.remove_navigation,
+            remove_forms=self.remove_forms,
+        )
+
+        keep_inline_images_in = list(self.keep_inline_images_in) if self.keep_inline_images_in else []
+        strip_tags = list(self.strip_tags) if self.strip_tags else []
+
+        options = HTMLToMarkdownConversionOptions(
+            heading_style=self.heading_style,
+            list_indent_type=self.list_indent_type,
+            list_indent_width=self.list_indent_width,
+            bullets=self.bullets,
+            strong_em_symbol=self.strong_em_symbol,
+            escape_asterisks=self.escape_asterisks,
+            escape_underscores=self.escape_underscores,
+            escape_misc=self.escape_misc,
+            escape_ascii=self.escape_ascii,
+            code_language=self.code_language,
+            autolinks=self.autolinks,
+            default_title=self.default_title,
+            keep_inline_images_in=keep_inline_images_in,
+            br_in_tables=self.br_in_tables,
+            highlight_style=self.highlight_style,
+            extract_metadata=self.extract_metadata,
+            whitespace_mode=self.whitespace_mode,
+            strip_newlines=self.strip_newlines,
+            wrap=self.wrap,
+            wrap_width=self.wrap_width,
+            convert_as_inline=self.convert_as_inline,
+            sub_symbol=self.sub_symbol,
+            sup_symbol=self.sup_symbol,
+            newline_style=self.newline_style,
+            code_block_style=self.code_block_style,
+            strip_tags=strip_tags,
+            debug=self.debug,
+            encoding=self.encoding,
+        )
+
+        options.preprocessing = preprocessing
+        return options, preprocessing
+
+    def to_dict(self, include_none: bool = False) -> dict[str, Any]:
         result = msgspec.to_builtins(self, builtin_types=(type(None),), order="deterministic")
+        if result.get("keep_inline_images_in") is not None:
+            result["keep_inline_images_in"] = list(result["keep_inline_images_in"])
+        if result.get("strip_tags") is not None:
+            result["strip_tags"] = list(result["strip_tags"])
+        if result.get("convert") is not None:
+            result["convert"] = list(result["convert"])
+
+        if include_none:
+            return result  # type: ignore[no-any-return]
+
         return {k: v for k, v in result.items() if v is not None}
+
+
+@dataclass(unsafe_hash=True, frozen=True, slots=True)
+class TokenReductionConfig:
+    mode: Literal["off", "light", "moderate"] = "off"
+    preserve_markdown: bool = True
+    custom_stopwords: dict[str, list[str]] | None = field(default=None, compare=False, hash=False)
+    language_hint: str | None = None
+
+    def __post_init__(self) -> None:
+        if self.language_hint:
+            hint = self.language_hint.strip()
+
+            if not hint or len(hint) > 50 or any(c in hint for c in "\x00\r\n\t"):
+                object.__setattr__(self, "language_hint", None)
+                return
+
+            try:
+                normalized = langcodes.standardize_tag(hint)
+
+                lang = langcodes.Language.get(normalized).language
+
+                if lang and lang != hint:
+                    object.__setattr__(self, "language_hint", lang)
+            except (ValueError, AttributeError, TypeError):
+                object.__setattr__(self, "language_hint", None)
