@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import multiprocessing as mp
+import traceback
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from pathlib import Path
 from typing import TYPE_CHECKING, Final, cast
@@ -10,17 +11,19 @@ import anyio
 from kreuzberg._chunker import get_chunker
 from kreuzberg._document_classification import auto_detect_document_type
 from kreuzberg._entity_extraction import extract_entities, extract_keywords
+from kreuzberg._error_handling import safe_feature_execution, should_exception_bubble_up
 from kreuzberg._language_detection import detect_languages
 from kreuzberg._mime_types import (
     validate_mime_type,
 )
 from kreuzberg._registry import ExtractorRegistry
+from kreuzberg._token_reduction import get_reduction_stats, reduce_tokens
 from kreuzberg._types import ExtractionConfig, ExtractionResult
 from kreuzberg._utils._document_cache import get_document_cache
 from kreuzberg._utils._errors import create_error_context
 from kreuzberg._utils._string import safe_decode
 from kreuzberg._utils._sync import run_maybe_sync, run_sync_only
-from kreuzberg.exceptions import ValidationError
+from kreuzberg.exceptions import KreuzbergError, ValidationError
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
@@ -31,15 +34,6 @@ DEFAULT_CONFIG: Final[ExtractionConfig] = ExtractionConfig()
 
 
 async def _handle_cache_async(path: Path, config: ExtractionConfig) -> ExtractionResult | None:
-    """Handle cache lookup and coordination with other processes.
-
-    Args:
-        path: Path to the file being processed
-        config: Extraction configuration
-
-    Returns:
-        Cached result if available, None otherwise
-    """
     cache = get_document_cache()
 
     cached_result = cache.get(path, config)
@@ -47,7 +41,7 @@ async def _handle_cache_async(path: Path, config: ExtractionConfig) -> Extractio
         return cached_result
 
     if cache.is_processing(path, config):
-        event = cache.mark_processing(path, config)
+        event = cache.mark_processing(path, config)  # pragma: no cover
         await anyio.to_thread.run_sync(event.wait)  # pragma: no cover
 
         return cache.get(path, config)  # pragma: no cover
@@ -58,39 +52,107 @@ async def _handle_cache_async(path: Path, config: ExtractionConfig) -> Extractio
 def _validate_and_post_process_helper(
     result: ExtractionResult, config: ExtractionConfig, file_path: Path | None = None
 ) -> ExtractionResult:
+    if result.metadata is None:
+        result.metadata = {}
+
     if config.chunk_content:
-        result.chunks = _handle_chunk_content(
-            mime_type=result.mime_type,
-            config=config,
-            content=result.content,
+        result.chunks = safe_feature_execution(
+            feature_name="chunking",
+            execution_func=lambda: _handle_chunk_content(
+                mime_type=result.mime_type,
+                config=config,
+                content=result.content,
+            ),
+            default_value=[],
+            result=result,
         )
 
     if config.extract_entities:
-        try:
-            result.entities = extract_entities(
+        result.entities = safe_feature_execution(
+            feature_name="entity_extraction",
+            execution_func=lambda: extract_entities(
                 result.content,
                 custom_patterns=config.custom_entity_patterns,
-            )
-        except RuntimeError:
-            result.entities = None
+            ),
+            default_value=None,
+            result=result,
+        )
 
     if config.extract_keywords:
-        try:
-            result.keywords = extract_keywords(
+        result.keywords = safe_feature_execution(
+            feature_name="keyword_extraction",
+            execution_func=lambda: extract_keywords(
                 result.content,
                 keyword_count=config.keyword_count,
-            )
-        except RuntimeError:
-            result.keywords = None
+            ),
+            default_value=None,
+            result=result,
+        )
 
     if config.auto_detect_language:
-        result.detected_languages = detect_languages(
-            result.content,
-            config=config.language_detection_config,
+
+        def _detect_language() -> list[str]:
+            lang_config = config.language_detection_config
+            if lang_config is None:
+                from kreuzberg._types import LanguageDetectionConfig  # noqa: PLC0415
+
+                lang_config = LanguageDetectionConfig(model=config.language_detection_model)
+
+            return detect_languages(result.content, config=lang_config) or []
+
+        result.detected_languages = safe_feature_execution(
+            feature_name="language_detection",
+            execution_func=_detect_language,
+            default_value=[],
+            result=result,
         )
 
     if config.auto_detect_document_type:
-        result = auto_detect_document_type(result, config, file_path=file_path)
+        result = safe_feature_execution(
+            feature_name="document_type_detection",
+            execution_func=lambda: auto_detect_document_type(result, config, file_path=file_path),
+            default_value=result,
+            result=result,
+        )
+
+    if config.token_reduction is not None and config.token_reduction.mode != "off":
+
+        def _apply_token_reduction() -> str:
+            original_content = result.content
+
+            language_hint = None
+            if result.detected_languages and len(result.detected_languages) > 0:
+                language_hint = result.detected_languages[0]
+
+            reduced_content = (
+                reduce_tokens(
+                    original_content,
+                    config=config.token_reduction,
+                    language=language_hint,
+                )
+                if config.token_reduction
+                else original_content
+            )
+            reduction_stats = get_reduction_stats(original_content, reduced_content)
+
+            if result.metadata is not None:
+                result.metadata["token_reduction"] = {
+                    "character_reduction_ratio": reduction_stats["character_reduction_ratio"],
+                    "token_reduction_ratio": reduction_stats["token_reduction_ratio"],
+                    "original_characters": reduction_stats["original_characters"],
+                    "reduced_characters": reduction_stats["reduced_characters"],
+                    "original_tokens": reduction_stats["original_tokens"],
+                    "reduced_tokens": reduction_stats["reduced_tokens"],
+                }
+
+            return reduced_content
+
+        result.content = safe_feature_execution(
+            feature_name="token_reduction",
+            execution_func=_apply_token_reduction,
+            default_value=result.content,
+            result=result,
+        )
 
     return result
 
@@ -103,8 +165,22 @@ async def _validate_and_post_process_async(
 
     result = _validate_and_post_process_helper(result, config, file_path)
 
-    for post_processor in config.post_processing_hooks or []:
-        result = await run_maybe_sync(post_processor, result)
+    for i, post_processor in enumerate(config.post_processing_hooks or []):
+        try:
+            result = await run_maybe_sync(post_processor, result)
+        except (KreuzbergError, ValueError, RuntimeError, TypeError) as e:  # noqa: PERF203
+            if result.metadata is None:
+                result.metadata = {}
+            error_list = result.metadata.setdefault("processing_errors", [])
+            if isinstance(error_list, list):
+                error_list.append(
+                    {
+                        "feature": f"post_processing_hook_{i}",
+                        "error_type": type(e).__name__,
+                        "error_message": str(e),
+                        "traceback": traceback.format_exc(),
+                    }
+                )
 
     return result
 
@@ -238,22 +314,18 @@ async def batch_extract_file(
                     config,
                 )
                 results[index] = result
-            except Exception as e:  # noqa: BLE001
-                error_result = ExtractionResult(
-                    content=f"Error: {type(e).__name__}: {e!s}",
-                    mime_type="text/plain",
-                    metadata={
-                        "error": f"{type(e).__name__}: {e!s}",
-                        "error_context": create_error_context(
-                            operation="batch_extract_file",
-                            file_path=str(path),
-                            error=e,
-                            index=index,
-                        ),
-                    },
-                    chunks=[],
+            except Exception as e:
+                if should_exception_bubble_up(e, "batch_processing"):
+                    raise
+
+                basic_result = _attempt_basic_extraction(
+                    None,
+                    None,
+                    e,
+                    index,
+                    file_path=str(path),
                 )
-                results[index] = error_result
+                results[index] = basic_result
 
     async with anyio.create_task_group() as tg:
         for i, path in enumerate(file_paths):
@@ -287,23 +359,12 @@ async def batch_extract_bytes(
             try:
                 result = await extract_bytes(content, mime_type, config)
                 results[index] = result
-            except Exception as e:  # noqa: BLE001
-                error_result = ExtractionResult(
-                    content=f"Error: {type(e).__name__}: {e!s}",
-                    mime_type="text/plain",
-                    metadata={
-                        "error": f"{type(e).__name__}: {e!s}",
-                        "error_context": create_error_context(
-                            operation="batch_extract_bytes",
-                            error=e,
-                            index=index,
-                            mime_type=mime_type,
-                            content_size=len(content),
-                        ),
-                    },
-                    chunks=[],
-                )
-                results[index] = error_result
+            except Exception as e:
+                if should_exception_bubble_up(e, "batch_processing"):
+                    raise
+
+                basic_result = _attempt_basic_extraction(content, mime_type, e, index)
+                results[index] = basic_result
 
     async with anyio.create_task_group() as tg:
         for i, (content, mime_type) in enumerate(contents):
@@ -312,6 +373,125 @@ async def batch_extract_bytes(
     return results
 
 
+def _attempt_basic_extraction(
+    content: bytes | None, mime_type: str | None, original_error: Exception, index: int, *, file_path: str | None = None
+) -> ExtractionResult:
+    """Attempt basic extraction when full extraction fails, preserving as much as possible.
+
+    This function tries to extract at least basic text content even when advanced
+    features like OCR, entity extraction, etc. fail.
+
+    Args:
+        content: The raw content bytes (None for file extractions)
+        mime_type: The MIME type of the content (None if unknown)
+        original_error: The exception that caused the main extraction to fail
+        index: Index of this content in the batch
+        file_path: Optional file path for file-based extractions
+
+    Returns:
+        A basic ExtractionResult with whatever could be extracted
+    """
+    if (
+        isinstance(original_error, (ValueError, TypeError, ValidationError))
+        or "mock" in str(type(original_error)).lower()
+    ):
+        return ExtractionResult(
+            content=f"Error: {type(original_error).__name__}: {original_error!s}",
+            mime_type="text/plain",
+            metadata={
+                "error": f"{type(original_error).__name__}: {original_error!s}",
+                "error_context": create_error_context(
+                    operation="batch_extract_file" if file_path else "batch_extract_bytes",
+                    error=original_error,
+                    index=index,
+                    mime_type=mime_type,
+                    content_size=len(content) if content else 0,
+                    file_path=file_path,
+                ),
+            },
+            chunks=[],
+            entities=[],
+            keywords=[],
+            detected_languages=[],
+            tables=[],
+            images=[],
+            image_ocr_results=[],
+        )
+
+    try:
+        if content is None:
+            return ExtractionResult(
+                content=f"Error: {type(original_error).__name__}: {original_error!s}",
+                mime_type="text/plain",
+                metadata={
+                    "error": f"{type(original_error).__name__}: {original_error!s}",
+                    "error_context": create_error_context(
+                        operation="batch_extract_file",
+                        error=original_error,
+                        index=index,
+                        file_path=file_path,
+                    ),
+                },
+                chunks=[],
+                entities=[],
+                keywords=[],
+                detected_languages=[],
+                tables=[],
+                images=[],
+                image_ocr_results=[],
+            )
+
+        mime_type = validate_mime_type(mime_type=mime_type)
+        if extractor := ExtractorRegistry.get_extractor(mime_type=mime_type, config=ExtractionConfig()):
+            basic_result = extractor.extract_bytes_sync(content)
+
+            if basic_result.metadata is None:
+                basic_result.metadata = {}
+
+            basic_result.metadata["extraction_error"] = {
+                "error_type": type(original_error).__name__,
+                "error_message": str(original_error),
+                "traceback": traceback.format_exc(),
+                "context": create_error_context(
+                    operation="batch_extract_file" if file_path else "batch_extract_bytes",
+                    error=original_error,
+                    index=index,
+                    mime_type=mime_type,
+                    content_size=len(content),
+                    file_path=file_path,
+                ),
+                "recovery_mode": "basic_extraction",
+            }
+
+            return basic_result
+
+    except (KreuzbergError, ValueError, RuntimeError, TypeError):
+        pass
+
+    return ExtractionResult(
+        content=f"Error: {type(original_error).__name__}: {original_error!s}",
+        mime_type="text/plain",
+        metadata={
+            "error": f"{type(original_error).__name__}: {original_error!s}",
+            "error_context": create_error_context(
+                operation="batch_extract_file" if file_path else "batch_extract_bytes",
+                error=original_error,
+                index=index,
+                mime_type=mime_type,
+                content_size=len(content) if content else 0,
+                file_path=file_path,
+            ),
+        },
+        chunks=[],
+        entities=[],
+        keywords=[],
+        detected_languages=[],
+        tables=[],
+        images=[],
+        image_ocr_results=[],
+    )
+
+
 def extract_bytes_sync(content: bytes, mime_type: str, config: ExtractionConfig = DEFAULT_CONFIG) -> ExtractionResult:
     """Synchronous version of extract_bytes.
 
@@ -362,7 +542,7 @@ def extract_file_sync(
             return cached_result
 
         if cache.is_processing(path, config):
-            event = cache.mark_processing(path, config)
+            event = cache.mark_processing(path, config)  # pragma: no cover
             event.wait()  # pragma: no cover
 
             # Try cache again after waiting for other process to complete  # ~keep
@@ -422,21 +602,18 @@ def batch_extract_file_sync(
                 index,
                 extract_file_sync(file_path=Path(file_path), mime_type=None, config=config),
             )
-        except Exception as e:  # noqa: BLE001
-            error_result = ExtractionResult(
-                content=f"Error: {type(e).__name__}: {e!s}",
-                mime_type="text/plain",
-                metadata={
-                    "error": f"{type(e).__name__}: {e!s}",
-                    "error_context": create_error_context(
-                        operation="batch_extract_file_sync",
-                        file_path=str(file_path),
-                        error=e,
-                    ),
-                },
-                chunks=[],
+        except Exception as e:
+            if should_exception_bubble_up(e, "batch_processing"):
+                raise
+
+            basic_result = _attempt_basic_extraction(
+                None,
+                None,
+                e,
+                index,
+                file_path=str(file_path),
             )
-            return (index, error_result)
+            return (index, basic_result)
 
     with ThreadPoolExecutor(max_workers=max_workers) as executor:
         future_to_index = {executor.submit(extract_single, i, fp): i for i, fp in enumerate(file_paths)}
@@ -472,23 +649,12 @@ def batch_extract_bytes_sync(
         """Extract single content with index for ordering."""
         try:
             return (index, extract_bytes_sync(content=content, mime_type=mime_type, config=config))
-        except Exception as e:  # noqa: BLE001
-            error_result = ExtractionResult(
-                content=f"Error: {type(e).__name__}: {e!s}",
-                mime_type="text/plain",
-                metadata={
-                    "error": f"{type(e).__name__}: {e!s}",
-                    "error_context": create_error_context(
-                        operation="batch_extract_bytes_sync",
-                        error=e,
-                        index=index,
-                        mime_type=mime_type,
-                        content_size=len(content),
-                    ),
-                },
-                chunks=[],
-            )
-            return (index, error_result)
+        except Exception as e:
+            if should_exception_bubble_up(e, "batch_processing"):
+                raise
+
+            basic_result = _attempt_basic_extraction(content, mime_type, e, index)
+            return (index, basic_result)
 
     with ThreadPoolExecutor(max_workers=max_workers) as executor:
         future_to_index = {
